﻿using System;
using System.ComponentModel;
using System.Runtime.CompilerServices;

using Microsoft.AppCenter;
using Microsoft.AppCenter.Analytics;

using Windows.UI.Xaml.Controls;
using Windows.UI.Popups;

namespace ShutdownTimerUWP.Views
{
    public sealed partial class TimerPage : Page, INotifyPropertyChanged
    {
        public TimerPage()
        {
            InitializeComponent();
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void Set<T>(ref T storage, T value, [CallerMemberName]string propertyName = null)
        {
            if (Equals(storage, value))
            {
                return;
            }

            storage = value;
            OnPropertyChanged(propertyName);
        }

        private void OnPropertyChanged(string propertyName) => PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));

        private void Start_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            // Initialize variables
            bool use_countdown = true;
            bool use_notifications = true;
            TimeSpan time;

            // Check couting method radio buttons
            if (Countdown_Radio.IsChecked == true) { use_countdown = true; }
            else if (Timer_Radio.IsChecked == true) { use_countdown = false; }

            // Check notification checkbox
            if (Notifications_CheckBox.IsChecked == true) { use_notifications = true; }
            else if (Notifications_CheckBox.IsChecked == false) { use_notifications = false; }

            // Get time
            time = Time_Picker.Time;
        }

        private async void Method_Help_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            var dialog = new MessageDialog("Using Coutdown will count down from the time you selected to zero. " +
                "So if you put 2 minutes in the selection, the timer will fire after 2 minutes.\n\n" +
                "But if you use the Time method, the timer will wait for the time to come so in the case of the time beeing 00:02 like above, " +
                "the timer will fire two minutes after midnight.");
            dialog.Title = "Help";
            dialog.Commands.Add(new UICommand { Label = "Ok", Id = 0 });
            await dialog.ShowAsync();
        }
    }
}
