﻿using ShutdownTimer.Core.Helpers;

using System;
using System.ComponentModel;
using System.Runtime.CompilerServices;

using Windows.UI.Xaml.Controls;

namespace ShutdownTimer.Views
{
    public sealed partial class SetTimerPage : Page, INotifyPropertyChanged
    {
        public SetTimerPage()
        {
            InitializeComponent();
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void Set<T>(ref T storage, T value, [CallerMemberName]string propertyName = null)
        {
            if (Equals(storage, value))
            {
                return;
            }

            storage = value;
            OnPropertyChanged(propertyName);
        }

        private void OnPropertyChanged(string propertyName) => PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));

        #region "UI buttons"

        private void sheduleShutdownButton_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            StartCountdown("shutdown");
        }

        private void sheduleHibernationButton_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            StartCountdown("hibernate");
        }

        private void sheduleSleepFlyout_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            StartCountdown("sleep");
        }

        private void sheduleLockFlyout_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            StartCountdown("lock");
        }

        private void sheduleLogoutFlyout_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            StartCountdown("logout");
        }

        private void sheduleRestartFlyout_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            StartCountdown("restart");
        }

        #endregion

        private void StartCountdown(string action)
        {
            if (Numeric.IsNumeric(HoursNumberBox.Text) && Numeric.IsNumeric(MinutesNumberBox.Text) && Numeric.IsNumeric(SecondsNumberBox.Text))
            {
                int temp_hours, temp_minutes, temp_seconds;
                (temp_hours, temp_minutes, temp_seconds) = Numeric.ConvertTime(Convert.ToInt32(HoursNumberBox.Text), Convert.ToInt32(MinutesNumberBox.Text), Convert.ToInt32(SecondsNumberBox.Text));


                Clockwork.UpdateTime(
                    hours: temp_hours,
                    minutes: temp_minutes,
                    seconds: temp_seconds,
                    initialTime: true);
                Clockwork.SetAction(action: action);
                this.Frame.Navigate(typeof(WatchTimerPage));
            }
        }
    }
}
